# Feature List

This page lists all of the features Tachi has.
Some games may utilise these better than others.

*****

## Sessions

Sessions are a feature in Tachi that group up your
scores depending on *when* they were achieved.

Sessions are meant to mimic the colloquial use of
the term - Players typically refer to their scores
as being part of a session, so why don't we show
their scores in that fashion aswell?

### What is a session?

A session is a group of scores that all happened
around the same time.

When you get a new score (and don't have a session nearby),
a session is automatically created!
Then, if you get a score within two hours of that last score,
that score is added to the session - this will repeat
until eventually you spend more than two hours between
your score and the last score.

In practice, the only time you're spending more than
two hours between a score is when you're either not playing
anymore, or taking a significant enough break that you're
probably no longer warm.

!!! info
	The reason we use the two-hour rule instead of just
	splitting on a day is that not everyone lives in the
	same timezone, and their session may get unexpectedly
	split in two!

### Why bother?

Since players use the term "sessions" so frequently, it
makes sense to let them view scores how they already
think about them.

Advantages of sessions also include being able to name and
categorise them - so you can search back on them in the
future.

You can also share a single link to your finished session,
instead of having to squish all your great scores into one 
long twitter thread!

Another small advantages include statistics - we can
calculate your "ability" that session, and graph it
over time - so you can see your real time improvement!

### What if I play multiple games?

**Sessions can only store scores of the same game and playtype**.

That is, if you have an IIDX SP session "ongoing", and get
an IIDX DP score - you will have two sessions ongoing at
the same time!

Your IIDX DP score will **not** be added to your SP session.

!!! note
	For home players, this is likely to not ever be an
	issue, but for arcade players - where people typically
	throw a couple credits into different games every day,
	it can become a problem!

### What does this performance statistic mean?

You can read all about session statistics [here](./stats/tachi.md#session-ratings).

### Summary

That's it for sessions! As a quick summary:

- Sessions group your scores based on when they've happened.
- Sessions are easy to share with other players.
- Sessions are a nice way of displaying all the scores you did!

*****

## Goals

!!! note
	This feature goes hand-in-hand with [Quests](#quests).

Goals are a built-in way of setting targets for yourself.

A lot of the time, people have goals they're aiming for in
rhythm games, but at the moment, people generally have to
keep said goals in their head.

Tachi lets you set and track your own goals - so you don't
have to remember them!

### Advantages

With your goals set and stored, we can see other users with
similar goals - which might give you good ideas for other
goals to aim for.

You can also be automatically notified when a goal is
achieved, or when you've made some progress towards it.

If you've ever been in the middle of a session and not
known what to play, you could always look at your list of
goals and try to check off some older ones, too!

### What kind of goals can I set?

There are two different parameters that control a goal.

#### Criteria

The criteria determines what our goal actually is.

You can set goals for Lamps, Grades, Percents and score.

!!! example
	Get 950'000 on FREEDOM DiVE.

	SS FREEDOM DiVE.

	Clear FREEDOM DiVE

	Get 95% on FREEDOM DiVE.

We can also control *how many* scores need to match the
given criteria. This makes more sense when goals apply
to more than one chart.

!!! example
	AAA 50 Charts in the Level 12 folder.

	FULL COMBO 10% of the Level 11 folder.

	Clear either FREEDOM DiVE or Blue Zenith.

#### Charts

You can also control what charts your goal applies to.

The most simple option is to only select one chart.

!!! example
	Full Combo xi - FREEDOM DiVE (FOUR DIMENSIONS)

You can also select multiple, fixed charts.

!!! example
	Full Combo xi - FREEDOM DiVE or NOMA - BRAIN POWER.
	
	Get 95% on 2 of the following, FREEDOM DiVE, Elemental Creation, BRAIN POWER or Blue Zenith.

Alternatively, you can set goals on folders.

!!! example
	Full combo any chart in the Level 12 folder.

	Clear 50% of the Level 12 folder.

	AAA 100 charts in the Level 12 folder.

Or, remove the chart restriction entirely!

!!! example
	AAA any chart.

	AAA 100 charts.

	Full Combo Every Chart.

### Summary

That's it for goals! As a quick summary:

- Goals can be set for charts, folders, or anything!
- You can set goals for lamps, scores, percents and grades.
- You can share your goals with other users, and see what your rivals have set!

This is nice, but there's something missing...

## Quests

As mentioned above, Goals were designed to go hand-in-hand
with this feature.

### Issues with Just Goals.

The main issue with goals on their own is that we're not
good at setting our own goals.

What if someone doesn't *really* know what they should be
aiming for, or is just lazy and doesn't like setting goals?

In general, we're bad at setting our own goals. How can
we fix that?

### What are Quests?

Quests are **pre-made groups of goals**. As an example,
we might bundle together some goals aimed at SOUND VOLTEX
11 dan players. You can then subscribe to that quest,
and all those goals will be merged with your list of goals!

This solves the problem of having to set and manage *loads*
of goals on your own - since other people can come up with
and debate good goals for your skill level, and you can
just seamlessly integrate them with your play!

### Who makes the Quests?

The community! People are free to create their own
quests and share them.

### Why are they called Quests?

Quests work as - well - quests!

They're achievable, just like goals, which means the
quest creator can set criteria.

A basic example would be counting the quest as
achieved when you've achieved *all* of the goals in that
quest.

More advanced options include achieving X goals inside a
quest, or X% of the goals inside a quest.

### Summary

That's all for quests! As a quick summary:

- Since quests are achievables, you'll be automatically notified when you tick another quest off!
- You can also see other user's quests and their progress.
- Quests reduce the pain of having to set your goals yourself!
